from discord.ext import commands
from typing import Union
import asyncio
import aiohttp

from .modules import Bots, Servers, Users


class BoticordClient:

    """
    This class is used to make it much easier to use the Boticord API.

    Parameters
    ----------
        bot : :class:`commands.Bot` | :class:`commands.AutoShardedBot`
            The discord.py Bot instance
        token : :class:`str`
            boticord api key

    Attributes
    ----------
        Bots : :class:`modules.bots.Bots`
            :class:`modules.bots.Bots` with all arguments filled.
        Servers : :class:`modules.servers.Servers`
           :class:`modules.servers.Servers` with all arguments filled.
        Users : :class:`modules.users.Users`
            :class:`modules.users.Users` with all arguments filled.
    """

    __slots__ = (
        "Bots",
        "Servers",
        "Users"
    )

    bot: Union[commands.Bot, commands.AutoShardedBot]

    def __init__(self, bot, token=None, **kwargs):
        loop = kwargs.get('loop') or asyncio.get_event_loop()
        session = kwargs.get('session') or aiohttp.ClientSession(loop=loop)
        self.Bots = Bots(bot, token=token, loop=loop, session=session)
        self.Servers = Servers(bot, token=token, loop=loop, session=session)
        self.Users = Users(token=token, loop=loop, session=session)



from . import exceptions


class Config:
    local_api = "https://boticord.top/api"
    general_api = "https://api.boticord.top/v1"
    http_exceptions = {401: exceptions.Unauthorized,
                       403: exceptions.Forbidden,
                       404: exceptions.NotFound,
                       429: exceptions.ToManyRequests,
                       500: exceptions.ServerError,
                       503: exceptions.ServerError}



class BoticordException(Exception):
    """Base exception class for boticordpy.
    This could be caught to handle any exceptions thrown from this library.
    """


class HTTPException(BoticordException):
    """Exception that's thrown when an HTTP request operation fails.

    Attributes
    ----------
    response:
        The response of the failed HTTP request.
    message:
        The text of the error. Could be an empty string.
    """

    def __init__(self, response, message):
        self.response = response
        if isinstance(message, dict):
            self.text = message.get('message', '')
            self.code = message.get('code', 0)
        else:
            self.text = message

        fmt = f"{self.response.reason} (Status code: {self.response.status})"
        if self.text:
            fmt = f"{fmt}: {self.text}"

        super().__init__(fmt)


class Unauthorized(HTTPException):
    """Exception that's thrown when status code 401 occurs."""


class Forbidden(HTTPException):
    """Exception that's thrown when status code 403 occurs."""


class NotFound(HTTPException):
    """Exception that's thrown when status code 404 occurs."""


class ToManyRequests(HTTPException):
    """Exception that's thrown when status code 429 occurs."""


class ServerError(HTTPException):
    """Exception that's thrown when status code 500 or 503 occurs."""
